# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/matr',
                'doc_host': 'https://dsm-72.github.io',
                'git_url': 'https://github.com/dsm-72/matr',
                'lib_path': 'matr'},
  'syms': { 'matr.core': { 'matr.core.ImportedAttribute': ('core.html#importedattribute', 'matr/core.py'),
                           'matr.core.Spec': ('core.html#spec', 'matr/core.py'),
                           'matr.core.Spec.asdict': ('core.html#spec.asdict', 'matr/core.py'),
                           'matr.core.Spec.getvals': ('core.html#spec.getvals', 'matr/core.py'),
                           'matr.core.Spec.setvals': ('core.html#spec.setvals', 'matr/core.py'),
                           'matr.core.iattr': ('core.html#iattr', 'matr/core.py'),
                           'matr.core.iattrs': ('core.html#iattrs', 'matr/core.py'),
                           'matr.core.matr': ('core.html#matr', 'matr/core.py'),
                           'matr.core.matr.__asset_repr__': ('core.html#matr.__asset_repr__', 'matr/core.py'),
                           'matr.core.matr.__call__': ('core.html#matr.__call__', 'matr/core.py'),
                           'matr.core.matr.__getattribute__': ('core.html#matr.__getattribute__', 'matr/core.py'),
                           'matr.core.matr.__getitem__': ('core.html#matr.__getitem__', 'matr/core.py'),
                           'matr.core.matr.__import__': ('core.html#matr.__import__', 'matr/core.py'),
                           'matr.core.matr.__init__': ('core.html#matr.__init__', 'matr/core.py'),
                           'matr.core.matr.__repr__': ('core.html#matr.__repr__', 'matr/core.py'),
                           'matr.core.matr.ib': ('core.html#matr.ib', 'matr/core.py'),
                           'matr.core.matr.io': ('core.html#matr.io', 'matr/core.py'),
                           'matr.core.matrs': ('core.html#matrs', 'matr/core.py'),
                           'matr.core.pmod': ('core.html#pmod', 'matr/core.py'),
                           'matr.core.pmod.__dir__': ('core.html#pmod.__dir__', 'matr/core.py'),
                           'matr.core.pmod.__getattr__': ('core.html#pmod.__getattr__', 'matr/core.py'),
                           'matr.core.pmod.__init__': ('core.html#pmod.__init__', 'matr/core.py'),
                           'matr.core.pmod.__setattr__': ('core.html#pmod.__setattr__', 'matr/core.py'),
                           'matr.core.pmod.attrs': ('core.html#pmod.attrs', 'matr/core.py'),
                           'matr.core.pmod.names': ('core.html#pmod.names', 'matr/core.py'),
                           'matr.core.pmod.populate': ('core.html#pmod.populate', 'matr/core.py')},
            'matr.inst': {}}}
